@@ -63,11 +63,13 @@ rloop timeoutIO mlog host handler inputStr
                       (topStr, restStr) <- required "failed to separate request" $ splitAtEmptyLine inputStr
                       (rql, headerStr)  <- required "failed to separate headers/body" $ splitAtCRLF topStr
                       let (m,u,v) = requestLine rql
-                      headers' <- parseHeaders "host" (L.unpack headerStr)
+                      headers' <- case parseHeaders "host" (L.unpack headerStr) of
+                        Nothing -> Left "failed to parse host header"
+                        Just x -> Right x
                       let headers = mkHeaders headers'
                       let contentLen = fromMaybe 0 $ fmap fst (P.readInt =<< getHeaderUnsafe contentlengthC headers)
                       (body, nextRequest) <- case () of
-                          () | contentLen < 0               -> fail "negative content-length"
+                          () | contentLen < 0               -> Left "negative content-length"
                              | isJust $ getHeaderBS transferEncodingC headers ->
                                  return $ consumeChunks restStr
                              | otherwise                       -> return (L.splitAt (fromIntegral contentLen) restStr)
@@ -139,7 +141,9 @@ parseResponse inputStr =
        (rsl,headerStr) <- required "failed to separate headers/body" $
                           splitAtCRLF topStr
        let (_,code) = responseLine rsl
-       headers' <- parseHeaders "host" (L.unpack headerStr)
+       headers' <- case parseHeaders "host" (L.unpack headerStr) of
+         Nothing -> Left "failed to parse host header"
+         Just x -> Right x
        let headers = mkHeaders headers'
        let mbCL = fmap fst (B.readInt =<< getHeader "content-length" headers)
        (body,_) <-
